@@ -27,7 +27,6 @@ from gettext import gettext as _
 
 from gi.repository import Gdk, GObject, Gtk
 
-from gnomemusic.player import PlayerPlaylist
 from gnomemusic.search import Search
 from gnomemusic.widgets.albumcover import AlbumCover
 from gnomemusic.widgets.albumwidget import AlbumWidget
@@ -58,6 +57,8 @@ class SearchView(Gtk.Stack):
     search_state = GObject.Property(type=int, default=Search.State.NONE)
     selection_mode = GObject.Property(type=bool, default=False)
     state = GObject.Property(type=int, default=State.MAIN)
+    title = GObject.Property(
+        type=str, default="", flags=GObject.ParamFlags.READABLE)
 
     _album_header = Gtk.Template.Child()
     _album_flowbox = Gtk.Template.Child()
@@ -79,9 +80,7 @@ class SearchView(Gtk.Stack):
         """
         super().__init__(transition_type=Gtk.StackTransitionType.CROSSFADE)
 
-        # FIXME: Make these properties.
-        self.name = "search"
-        self.title = None
+        self.props.name = "search"
 
         self._application = application
         self._coremodel = application.props.coremodel
@@ -257,10 +256,7 @@ class SearchView(Gtk.Stack):
         (_, button) = event.get_button()
         if (button == Gdk.BUTTON_PRIMARY
                 and not self.props.selection_mode):
-            # self.emit('song-activated', widget)
-
-            self._coremodel.set_player_model(
-                PlayerPlaylist.Type.SEARCH_RESULT, self._model)
+            self._coremodel.props.active_core_object = widget.props.coresong
             self._player.play(widget.props.coresong)
 
         return True
@@ -361,7 +357,7 @@ class SearchView(Gtk.Stack):
             return
 
         artist_albums_widget = ArtistAlbumsWidget(
-            coreartist, self._application, True)
+            coreartist, self._application)
         # FIXME: Recreating a view here. Alternate solution is used
         # in AlbumsView: one view created and an update function.
         # Settle on one design.
