@@ -25,12 +25,11 @@
 from gettext import gettext as _
 from gi.repository import Gdk, GObject, Gtk
 
-from gnomemusic import log
 from gnomemusic.widgets.songwidget import SongWidget
 
 
 @Gtk.Template(resource_path='/org/gnome/Music/ui/DiscBox.ui')
-class DiscBox(Gtk.Box):
+class DiscBox(Gtk.ListBoxRow):
     """A widget which compromises one disc
 
     DiscBox contains a disc label for the disc number on top
@@ -46,41 +45,26 @@ class DiscBox(Gtk.Box):
     }
 
     selection_mode = GObject.Property(type=bool, default=False)
-    selection_mode_allowed = GObject.Property(type=bool, default=True)
     show_disc_label = GObject.Property(type=bool, default=False)
-    show_durations = GObject.Property(type=bool, default=False)
-    show_favorites = GObject.Property(type=bool, default=False)
-    show_song_numbers = GObject.Property(type=bool, default=False)
 
-    def __repr__(self):
-        return '<DiscBox>'
-
-    @log
-    def __init__(self, model):
+    def __init__(self, coredisc):
         """Initialize
 
-        :param model: The Gio.ListStore to use
+        :param CoreDisc coredisc: The CoreDisc object to use
         """
         super().__init__()
 
-        self._model = model
+        self._model = coredisc.props.model
+
+        disc_nr = coredisc.props.disc_nr
+        self._disc_label.props.label = _("Disc {}").format(disc_nr)
 
         self.bind_property(
             'show-disc-label', self._disc_label, 'visible',
             GObject.BindingFlags.SYNC_CREATE)
 
-        self._selection_mode_allowed = True
-
         self._list_box.bind_model(self._model, self._create_widget)
 
-    @log
-    def set_disc_number(self, disc_number):
-        """Set the dics number to display
-
-        :param int disc_number: Disc number to display
-        """
-        self._disc_label.props.label = _("Disc {}").format(disc_number)
-
     def select_all(self):
         """Select all songs"""
         def child_select_all(child):
@@ -89,13 +73,13 @@ class DiscBox(Gtk.Box):
 
         self._list_box.foreach(child_select_all)
 
-    def select_none(self):
+    def deselect_all(self):
         """Deselect all songs"""
-        def child_select_none(child):
+        def child_deselect_all(child):
             song_widget = child.get_child()
             song_widget.props.coresong.props.selected = False
 
-        self._list_box.foreach(child_select_none)
+        self._list_box.foreach(child_deselect_all)
 
     def _create_widget(self, coresong):
         song_widget = SongWidget(coresong)
@@ -107,9 +91,13 @@ class DiscBox(Gtk.Box):
 
         song_widget.connect('button-release-event', self._song_activated)
 
-        return song_widget
+        row = Gtk.ListBoxRow()
+        row.props.activatable = False
+        row.props.selectable = False
+        row.add(song_widget)
+
+        return row
 
-    @log
     def _song_activated(self, widget, event):
         if widget.props.select_click:
             widget.props.select_click = False
@@ -117,8 +105,7 @@ class DiscBox(Gtk.Box):
 
         mod_mask = Gtk.accelerator_get_default_mod_mask()
         if ((event.get_state() & mod_mask) == Gdk.ModifierType.CONTROL_MASK
-                and not self.props.selection_mode
-                and self.props.selection_mode_allowed):
+                and not self.props.selection_mode):
             self.props.selection_mode = True
             widget.props.select_click = True
             widget.props.coresong.props.selected = True
@@ -143,53 +130,25 @@ class DiscListBox(Gtk.ListBox):
     """
     __gtype_name__ = 'DiscListBox'
 
-    selection_mode_allowed = GObject.Property(type=bool, default=False)
+    selection_mode = GObject.Property(type=bool, default=False)
 
-    def __repr__(self):
-        return '<DiscListBox>'
-
-    @log
     def __init__(self):
         """Initialize"""
         super().__init__()
 
         self.props.valign = Gtk.Align.START
-        self._selection_mode = False
-
         self.get_style_context().add_class("disc-list-box")
 
-    @log
     def select_all(self):
         """Select all songs"""
         def child_select_all(child):
-            child.get_child().select_all()
+            child.select_all()
 
         self.foreach(child_select_all)
 
-    @log
-    def select_none(self):
+    def deselect_all(self):
         """Deselect all songs"""
-        def child_select_none(child):
-            child.get_child().select_none()
+        def child_deselect_all(child):
+            child.deselect_all()
 
-        self.foreach(child_select_none)
-
-    @GObject.Property(type=bool, default=False)
-    def selection_mode(self):
-        """selection mode getter
-
-        :returns: If selection mode is active
-        :rtype: bool
-        """
-        return self._selection_mode
-
-    @selection_mode.setter
-    def selection_mode(self, value):
-        """selection-mode setter
-
-        :param bool value: Activate selection mode
-        """
-        if not self.props.selection_mode_allowed:
-            return
-
-        self._selection_mode = value
+        self.foreach(child_deselect_all)
