@@ -25,7 +25,6 @@
 from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
-from gnomemusic import log
 from gnomemusic.gstplayer import Playback
 from gnomemusic.mediakeys import MediaKeys
 from gnomemusic.player import RepeatMode
@@ -40,15 +39,12 @@ from gnomemusic.views.songsview import SongsView
 from gnomemusic.views.playlistsview import PlaylistsView
 from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.notificationspopup import NotificationsPopup  # noqa
-from gnomemusic.widgets.playertoolbar import PlayerToolbar
+from gnomemusic.widgets.playertoolbar import PlayerToolbar  # noqa: F401
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
 from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
 from gnomemusic.widgets.selectiontoolbar import SelectionToolbar  # noqa: F401
 from gnomemusic.windowplacement import WindowPlacement
 
-import logging
-logger = logging.getLogger(__name__)
-
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/Window.ui")
 class Window(Gtk.ApplicationWindow):
@@ -56,19 +52,16 @@ class Window(Gtk.ApplicationWindow):
     __gtype_name__ = "Window"
 
     active_view = GObject.Property(type=GObject.GObject, default=None)
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
+    selected_songs_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
     notifications_popup = Gtk.Template.Child()
-    _box = Gtk.Template.Child()
+    _headerbar_stack = Gtk.Template.Child()
     _overlay = Gtk.Template.Child()
+    _player_toolbar = Gtk.Template.Child()
     _selection_toolbar = Gtk.Template.Child()
     _stack = Gtk.Template.Child()
 
-    def __repr__(self):
-        return '<Window>'
-
-    @log
     def __init__(self, app):
         """Initialize the main window.
 
@@ -76,46 +69,42 @@ class Window(Gtk.ApplicationWindow):
         """
         super().__init__(application=app, title=_("Music"))
 
-        # Hack
         self._app = app
+        self._coreselection = app.props.coreselection
 
-        self._app._coreselection.bind_property(
-            "selected-items-count", self, "selected-items-count")
+        self._coreselection.bind_property(
+            "selected-songs-count", self, "selected-songs-count")
 
         self._settings = app.props.settings
         self.add_action(self._settings.create_action('repeat'))
         select_all = Gio.SimpleAction.new('select_all', None)
         select_all.connect('activate', self._select_all)
         self.add_action(select_all)
-        select_none = Gio.SimpleAction.new('select_none', None)
-        select_none.connect('activate', self._select_none)
-        self.add_action(select_none)
+        deselect_all = Gio.SimpleAction.new('deselect_all', None)
+        deselect_all.connect('activate', self._deselect_all)
+        self.add_action(deselect_all)
 
         self.set_size_request(200, 100)
         WindowPlacement(self)
 
-        self.prev_view = None
-        self.curr_view = None
+        self._current_view = None
         self._view_before_search = None
 
         self._player = app.props.player
+        self._search = app.props.search
 
+        self._startup_timeout_id = 0
         self._setup_view()
 
         MediaKeys(self._player, self)
 
-    @log
     def _setup_view(self):
-        self._search = Search()
-        self._headerbar_stack = Gtk.Stack()
-        transition_type = Gtk.StackTransitionType.CROSSFADE
-        self._headerbar_stack.props.transition_type = transition_type
-        self._headerbar = HeaderBar()
+        self._headerbar = HeaderBar(self._app)
+        self._headerbar.props.stack = self._stack
         self._search_headerbar = SearchHeaderBar(self._app)
         self._search_headerbar.props.stack = self._stack
         self._headerbar_stack.add_named(self._headerbar, "main")
         self._headerbar_stack.add_named(self._search_headerbar, "search")
-        self._headerbar_stack.props.name = "search"
 
         self._search.bind_property(
             "search-mode-active", self._headerbar, "search-mode-active",
@@ -132,24 +121,23 @@ class Window(Gtk.ApplicationWindow):
         self._search.connect(
             "notify::search-mode-active", self._on_search_mode_changed)
 
-        self._player_toolbar = PlayerToolbar()
         self._player_toolbar.props.player = self._player
 
         self._headerbar.connect(
             'back-button-clicked', self._switch_back_from_childview)
 
         self.bind_property(
-            'selected-items-count', self._headerbar, 'selected-items-count')
+            'selected-songs-count', self._headerbar, 'selected-songs-count')
         self.bind_property(
-            "selected-items-count", self._selection_toolbar,
-            "selected-items-count")
+            "selected-songs-count", self._selection_toolbar,
+            "selected-songs-count")
         self.bind_property(
             'selection-mode', self._headerbar, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL
             | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
-            "selected-items-count", self._search_headerbar,
-            "selected-items-count")
+            "selected-songs-count", self._search_headerbar,
+            "selected-songs-count")
         self.bind_property(
             "selection-mode", self._search_headerbar, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL
@@ -161,7 +149,7 @@ class Window(Gtk.ApplicationWindow):
             "selection-mode", self._selection_toolbar, "visible")
         self.connect("notify::selection-mode", self._on_selection_mode_changed)
 
-        self.views = [Gtk.Box()] * len(View)
+        self.views = [None] * len(View)
         # Create only the empty view at startup
         # if no music, switch to empty view and hide stack
         # if some music is available, populate stack with mainviews,
@@ -175,34 +163,27 @@ class Window(Gtk.ApplicationWindow):
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
-        self._box.pack_end(self._player_toolbar, False, False, 0)
-
-        self.set_titlebar(self._headerbar_stack)
-
         self._selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
         self._search.connect("notify::state", self._on_search_state_changed)
 
         self._headerbar.props.state = HeaderBar.State.MAIN
-        self._headerbar_stack.show_all()
 
         self._app.props.coremodel.connect(
             "notify::songs-available", self._on_songs_available)
 
-        self._app.props.coremodel.props.grilo.connect(
+        self._app.props.coregrilo.connect(
             "notify::tracker-available", self._on_tracker_available)
 
-        if self._app.props.coremodel.props.songs_available:
-            self._switch_to_player_view()
-        else:
-            self._switch_to_empty_view()
+        self._startup_timeout_id = GLib.timeout_add(
+            2000, self._on_songs_available, None, None)
 
-    @log
     def _switch_to_empty_view(self):
         did_initial_state = self._settings.get_boolean('did-initial-state')
 
-        state = self._app.props.coremodel.props.grilo.props.tracker_available
+        state = self._app.props.coregrilo.props.tracker_available
         empty_view = self.views[View.EMPTY]
+        empty_view.props.visible = True
         if state == TrackerState.UNAVAILABLE:
             empty_view.props.state = EmptyView.State.NO_TRACKER
         elif state == TrackerState.OUTDATED:
@@ -217,11 +198,15 @@ class Window(Gtk.ApplicationWindow):
 
     def _on_search_mode_changed(self, search, value):
         if self._search.props.search_mode_active:
-            self._headerbar_stack.set_visible_child_name("search")
+            self._headerbar_stack.props.visible_child_name = "search"
         else:
-            self._headerbar_stack.set_visible_child_name("main")
+            self._headerbar_stack.props.visible_child_name = "main"
 
     def _on_songs_available(self, klass, value):
+        if self._startup_timeout_id > 0:
+            GLib.source_remove(self._startup_timeout_id)
+            self._startup_timeout_id = 0
+
         if self._app.props.coremodel.props.songs_available:
             self._switch_to_player_view()
         else:
@@ -231,15 +216,13 @@ class Window(Gtk.ApplicationWindow):
         self.props.active_view = self._stack.props.visible_child
 
     def _on_tracker_available(self, klass, value):
-        grilo = self._app.props.coremodel.props.grilo
-        new_state = grilo.props.tracker_available
+        new_state = self._app.props.coregrilo.props.tracker_available
 
         if new_state != TrackerState.AVAILABLE:
             self._switch_to_empty_view()
 
         self._on_songs_available(None, None)
 
-    @log
     def _switch_to_player_view(self):
         self._settings.set_boolean('did-initial-state', True)
         self._on_notify_model_id = self._stack.connect(
@@ -256,40 +239,33 @@ class Window(Gtk.ApplicationWindow):
 
         self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
 
-        # FIXME: In case Grilo is already initialized before the views
-        # get created, they never receive a 'ready' signal to trigger
-        # population. To fix this another check was added to baseview
-        # to populate if grilo is ready at the end of init. For this to
-        # work however, the headerbar stack needs to be created and
-        # populated. This is done below, by binding headerbar.stack to
-        # to window._stack. For this to succeed, the stack needs to be
-        # filled with something: Gtk.Box.
-        # This is a bit of circular logic that needs to be fixed.
         self._headerbar.props.state = HeaderBar.State.MAIN
-        self._headerbar.props.stack = self._stack
 
-        self.views[View.ALBUM] = AlbumsView(self._app, self._player)
-        self.views[View.ARTIST] = ArtistsView(self._app, self._player)
-        self.views[View.SONG] = SongsView(self._app, self._player)
-        self.views[View.PLAYLIST] = PlaylistsView(self._app, self._player)
-        self.views[View.SEARCH] = SearchView(self._app, self._player)
+        # All views are created together, so if the album view is
+        # already initialized, assume the rest are as well.
+        if self.views[View.ALBUM] is not None:
+            return
 
-        selectable_views = [View.ALBUM, View.ARTIST, View.SONG, View.SEARCH]
-        for view in selectable_views:
-            self.views[view].bind_property(
-                'selected-items-count', self, 'selected-items-count')
+        self.views[View.ALBUM] = AlbumsView(self._app)
+        self.views[View.ARTIST] = ArtistsView(self._app)
+        self.views[View.SONG] = SongsView(self._app)
+        self.views[View.PLAYLIST] = PlaylistsView(self._app)
+        self.views[View.SEARCH] = SearchView(self._app)
 
         # empty view has already been created in self._setup_view starting at
         # View.ALBUM
         # empty view state is changed once album view is visible to prevent it
         # from being displayed during startup
         for i in self.views[View.ALBUM:]:
-            if i.title:
-                self._stack.add_titled(i, i.name, i.title)
+            if i.props.title:
+                self._stack.add_titled(i, i.props.name, i.props.title)
             else:
-                self._stack.add_named(i, i.name)
+                self._stack.add_named(i, i.props.name)
 
-        self._stack.set_visible_child(self.views[View.ALBUM])
+        # The "visible-child" notification ensures that the AlbumView
+        # appears as selected by the stack switcher on launch.
+        self._stack.props.visible_child = self.views[View.ALBUM]
+        self._stack.notify("visible-child")
 
         self.views[View.SEARCH].bind_property(
             "search-state", self._search, "state",
@@ -301,21 +277,18 @@ class Window(Gtk.ApplicationWindow):
             "search-mode-active", self.views[View.ALBUM],
             "search-mode-active", GObject.BindingFlags.SYNC_CREATE)
 
-    @log
     def _select_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
 
         self.props.active_view.select_all()
 
-    @log
-    def _select_none(self, action=None, param=None):
+    def _deselect_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
 
-        self.props.active_view.unselect_all()
+        self.props.active_view.deselect_all()
 
-    @log
     def _on_key_press(self, widget, event):
         modifiers = event.get_state() & Gtk.accelerator_get_default_mod_mask()
         (_, keyval) = event.get_keyval()
@@ -332,6 +305,7 @@ class Window(Gtk.ApplicationWindow):
             # Open search bar on Ctrl + F
             if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
+                    and not self.props.selection_mode
                     and self._headerbar.props.state != HeaderBar.State.SEARCH):
                 search_mode = self._search.props.search_mode_active
                 self._search.props.search_mode_active = not search_mode
@@ -344,8 +318,6 @@ class Window(Gtk.ApplicationWindow):
             # Play next on Ctrl + N
             if keyval == Gdk.KEY_n:
                 self._player.next()
-            if keyval == Gdk.KEY_q:
-                self.props.application.quit()
             # Toggle repeat on Ctrl + R
             if keyval == Gdk.KEY_r:
                 if self._player.props.repeat_mode == RepeatMode.SONG:
@@ -367,7 +339,7 @@ class Window(Gtk.ApplicationWindow):
         # Ctrl+Shift+<KEY>
         elif modifiers == shift_ctrl_mask:
             if keyval == Gdk.KEY_A:
-                self._select_none()
+                self._deselect_all()
         # Alt+<KEY>
         elif modifiers == mod1_mask:
             # Go back from child view on Alt + Left
@@ -399,8 +371,10 @@ class Window(Gtk.ApplicationWindow):
 
             child = self.props.active_view
             if (keyval == Gdk.KEY_Delete
-                    and child == self.views[View.PLAYLIST]):
-                self.views[View.PLAYLIST].remove_playlist()
+                    and child == self.views[View.PLAYLIST]
+                    and not self.views[View.PLAYLIST].rename_active):
+                self.activate_action("playlist_delete", None)
+
             # Close selection mode or search bar after Esc is pressed
             if keyval == Gdk.KEY_Escape:
                 if self.props.selection_mode:
@@ -416,30 +390,28 @@ class Window(Gtk.ApplicationWindow):
                 and (modifiers == shift_mask
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
+                and not self.props.selection_mode
                 and self._headerbar.props.state != HeaderBar.State.SEARCH):
             self._search.props.search_mode_active = True
 
-    @log
     def _on_back_button_pressed(self, gesture, n_press, x, y):
         self._headerbar.emit('back-button-clicked')
 
-    @log
     def _notify_mode_disconnect(self, data=None):
         self._player.stop()
         self.notifications_popup.terminate_pending()
         self._stack.disconnect(self._on_notify_model_id)
 
-    @log
     def _on_notify_mode(self, stack, param):
-        self.prev_view = self.curr_view
-        self.curr_view = self.props.active_view
+        previous_view = self._current_view
+        self._current_view = self.props.active_view
 
         # Disable search mode when switching view
         search_views = [self.views[View.EMPTY], self.views[View.SEARCH]]
-        if (self.curr_view in search_views
-                and self.prev_view not in search_views):
-            self._view_before_search = self.prev_view
-        elif (self.curr_view not in search_views
+        if (self._current_view in search_views
+                and previous_view not in search_views):
+            self._view_before_search = previous_view
+        elif (self._current_view not in search_views
                 and self._search.props.search_mode_active is True):
             self._search.props.search_mode_active = False
 
@@ -449,11 +421,10 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.EMPTY],
             self.views[View.PLAYLIST]
         ]
-        allowed = self.curr_view not in no_selection_mode
+        allowed = self._current_view not in no_selection_mode
         self._headerbar.props.selection_mode_allowed = allowed
         self._search_headerbar.props.selection_mode_allowed = allowed
 
-    @log
     def _toggle_view(self, view_enum):
         # TODO: The SEARCH state actually refers to the child state of
         # the search mode. This fixes the behaviour as needed, but is
@@ -464,16 +435,14 @@ class Window(Gtk.ApplicationWindow):
                 and not self._headerbar.props.state == HeaderBar.State.SEARCH):
             self._stack.set_visible_child(self.views[view_enum])
 
-    @log
     def _on_search_state_changed(self, klass, param):
         if (self._search.props.state != Search.State.NONE
                 or not self._view_before_search):
             return
 
         # Get back to the view before the search
-        self._stack.set_visible_child(self._view_before_search)
+        self._stack.props.visible_child = self._view_before_search
 
-    @log
     def _switch_back_from_childview(self, klass=None):
         if self.props.selection_mode:
             return
@@ -482,26 +451,25 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.ALBUM],
             self.views[View.SEARCH]
         ]
-        if self.curr_view in views_with_child:
-            self.curr_view._back_button_clicked(self.curr_view)
+        if self._current_view in views_with_child:
+            self._current_view._back_button_clicked(self._current_view)
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
         if (not self.props.selection_mode
                 and self._player.state == Playback.STOPPED):
             self._player_toolbar.hide()
 
-    @log
     def _on_add_to_playlist(self, widget):
         if self.props.active_view == self.views[View.PLAYLIST]:
             return
 
-        selected_songs = self._app._coreselection.props.selected_items
+        selected_songs = self._coreselection.props.selected_songs
 
         if len(selected_songs) < 1:
             return
 
-        playlist_dialog = PlaylistDialog(self)
+        playlist_dialog = PlaylistDialog(self._app)
+        playlist_dialog.props.transient_for = self
         if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
             playlist = playlist_dialog.props.selected_playlist
             playlist.add_songs(selected_songs)
@@ -509,7 +477,6 @@ class Window(Gtk.ApplicationWindow):
         self.props.selection_mode = False
         playlist_dialog.destroy()
 
-    @log
     def set_player_visible(self, visible):
         """Set PlayWidget action visibility
 
